@@ -1,6 +1,6 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
  | Copyright (C) 2005-2013, The Roundcube Dev Team                       |
@@ -16,7 +16,6 @@
  +-----------------------------------------------------------------------+
 */
 
-
 /**
  * Class for HTML code creation
  *
@@ -26,14 +25,15 @@
 class html
 {
     protected $tagname;
-    protected $attrib = array();
-    protected $allowed = array();
     protected $content;
+    protected $attrib  = array();
+    protected $allowed = array();
 
     public static $doctype = 'xhtml';
     public static $lc_tags = true;
-    public static $common_attrib = array('id','class','style','title','align','unselectable');
-    public static $containers = array('iframe','div','span','p','h1','h2','h3','ul','form','textarea','table','thead','tbody','tr','th','td','style','script');
+    public static $common_attrib = array('id','class','style','title','align','unselectable','tabindex','role');
+    public static $containers    = array('iframe','div','span','p','h1','h2','h3','ul','form','textarea','table','thead','tbody','tr','th','td','style','script');
+    public static $bool_attrib   = array('checked','multiple','disabled','selected','autofocus','readonly');
 
 
     /**
@@ -65,14 +65,16 @@ class html
      *
      * @param string $tagname Tag name
      * @param array  $attrib  Tag attributes as key/value pairs
-     * @param string $content Optinal Tag content (creates a container tag)
-     * @param array  $allowed_attrib List with allowed attributes, omit to allow all
+     * @param string $content Optional Tag content (creates a container tag)
+     * @param array  $allowed List with allowed attributes, omit to allow all
+     *
      * @return string The XHTML tag
      */
-    public static function tag($tagname, $attrib = array(), $content = null, $allowed_attrib = null)
+    public static function tag($tagname, $attrib = array(), $content = null, $allowed = null)
     {
-        if (is_string($attrib))
+        if (is_string($attrib)) {
             $attrib = array('class' => $attrib);
+        }
 
         $inline_tags = array('a','span','img');
         $suffix = $attrib['nl'] || ($content && $attrib['nl'] !== false && !in_array($tagname, $inline_tags)) ? "\n" : '';
@@ -81,15 +83,17 @@ class html
         if (isset($content) || in_array($tagname, self::$containers)) {
             $suffix = $attrib['noclose'] ? $suffix : '</' . $tagname . '>' . $suffix;
             unset($attrib['noclose'], $attrib['nl']);
-            return '<' . $tagname  . self::attrib_string($attrib, $allowed_attrib) . '>' . $content . $suffix;
+            return '<' . $tagname  . self::attrib_string($attrib, $allowed) . '>' . $content . $suffix;
         }
         else {
-            return '<' . $tagname  . self::attrib_string($attrib, $allowed_attrib) . '>' . $suffix;
+            return '<' . $tagname  . self::attrib_string($attrib, $allowed) . '>' . $suffix;
         }
     }
 
     /**
+     * Return DOCTYPE tag of specified type
      *
+     * @param string $type Document type (html5, xhtml, 'xhtml-trans, xhtml-strict)
      */
     public static function doctype($type)
     {
@@ -113,6 +117,7 @@ class html
      *
      * @param mixed  $attr Hash array with tag attributes or string with class name
      * @param string $cont Div content
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -121,6 +126,7 @@ class html
         if (is_string($attr)) {
             $attr = array('class' => $attr);
         }
+
         return self::tag('div', $attr, $cont, array_merge(self::$common_attrib, array('onclick')));
     }
 
@@ -129,6 +135,7 @@ class html
      *
      * @param mixed  $attr Hash array with tag attributes or string with class name
      * @param string $cont Paragraph content
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -137,6 +144,7 @@ class html
         if (is_string($attr)) {
             $attr = array('class' => $attr);
         }
+
         return self::tag('p', $attr, $cont, self::$common_attrib);
     }
 
@@ -144,6 +152,7 @@ class html
      * Derrived method to create <img />
      *
      * @param mixed $attr Hash array with tag attributes or string with image source (src)
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -152,8 +161,9 @@ class html
         if (is_string($attr)) {
             $attr = array('src' => $attr);
         }
+
         return self::tag('img', $attr + array('alt' => ''), null, array_merge(self::$common_attrib,
-            array('src','alt','width','height','border','usemap','onclick','onerror')));
+            array('src','alt','width','height','border','usemap','onclick','onerror','onload')));
     }
 
     /**
@@ -161,6 +171,7 @@ class html
      *
      * @param mixed  $attr Hash array with tag attributes or string with link location (href)
      * @param string $cont Link content
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -169,6 +180,7 @@ class html
         if (is_string($attr)) {
             $attr = array('href' => $attr);
         }
+
         return self::tag('a', $attr, $cont, array_merge(self::$common_attrib,
             array('href','target','name','rel','onclick','onmouseover','onmouseout','onmousedown','onmouseup')));
     }
@@ -178,6 +190,7 @@ class html
      *
      * @param mixed  $attr Hash array with tag attributes or string with class name
      * @param string $cont Tag content
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -186,6 +199,7 @@ class html
         if (is_string($attr)) {
             $attr = array('class' => $attr);
         }
+
         return self::tag('span', $attr, $cont, self::$common_attrib);
     }
 
@@ -194,6 +208,7 @@ class html
      *
      * @param mixed  $attr Hash array with tag attributes or string with 'for' attrib
      * @param string $cont Tag content
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -202,13 +217,16 @@ class html
         if (is_string($attr)) {
             $attr = array('for' => $attr);
         }
-        return self::tag('label', $attr, $cont, array_merge(self::$common_attrib, array('for')));
+
+        return self::tag('label', $attr, $cont, array_merge(self::$common_attrib,
+            array('for','onkeypress')));
     }
 
     /**
      * Derrived method to create <iframe></iframe>
      *
      * @param mixed $attr Hash array with tag attributes or string with frame source (src)
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -217,15 +235,17 @@ class html
         if (is_string($attr)) {
             $attr = array('src' => $attr);
         }
+
         return self::tag('iframe', $attr, $cont, array_merge(self::$common_attrib,
-            array('src','name','width','height','border','frameborder','onload')));
+            array('src','name','width','height','border','frameborder','onload','allowfullscreen')));
     }
 
     /**
      * Derrived method to create <script> tags
      *
-     * @param mixed $attr Hash array with tag attributes or string with script source (src)
+     * @param mixed  $attr Hash array with tag attributes or string with script source (src)
      * @param string $cont Javascript code to be placed as tag content
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -248,6 +268,8 @@ class html
     /**
      * Derrived method for line breaks
      *
+     * @param array $attrib Associative arry with tag attributes
+     *
      * @return string HTML code
      * @see html::tag()
      */
@@ -259,8 +281,9 @@ class html
     /**
      * Create string with attributes
      *
-     * @param array $attrib Associative arry with tag attributes
+     * @param array $attrib  Associative array with tag attributes
      * @param array $allowed List of allowed attributes
+     *
      * @return string Valid attribute string
      */
     public static function attrib_string($attrib = array(), $allowed = null)
@@ -283,10 +306,11 @@ class html
                 continue;
             }
 
-            // ignore not allowed attributes
+            // ignore not allowed attributes, except aria-* and data-*
             if (!empty($allowed)) {
                 $is_data_attr = @substr_compare($key, 'data-', 0, 5) === 0;
-                if (!isset($allowed_f[$key]) && (!$is_data_attr || !isset($allowed_f['data-*']))) {
+                $is_aria_attr = @substr_compare($key, 'aria-', 0, 5) === 0;
+                if (!$is_aria_attr && !$is_data_attr && !isset($allowed_f[$key])) {
                     continue;
                 }
             }
@@ -297,9 +321,14 @@ class html
             }
 
             // attributes with no value
-            if (in_array($key, array('checked', 'multiple', 'disabled', 'selected', 'autofocus'))) {
+            if (in_array($key, self::$bool_attrib)) {
                 if ($value) {
-                    $attrib_arr[] = $key . '="' . $key . '"';
+                    $value = $key;
+                    if (self::$doctype == 'xhtml') {
+                        $value .= '="' . $value . '"';
+                    }
+
+                    $attrib_arr[] = $value;
                 }
             }
             else {
@@ -313,20 +342,24 @@ class html
     /**
      * Convert a HTML attribute string attributes to an associative array (name => value)
      *
-     * @param string Input string
+     * @param string $str Input string
+     *
      * @return array Key-value pairs of parsed attributes
      */
     public static function parse_attrib_string($str)
     {
         $attrib = array();
-        $regexp = '/\s*([-_a-z]+)=(["\'])??(?(2)([^\2]*)\2|(\S+?))/Ui';
+        $html   = '<html>'
+            . '<head><meta http-equiv="Content-Type" content="text/html; charset=' . RCUBE_CHARSET . '" /></head>'
+            . '<body><div ' . rtrim($str, '/ ') . ' /></body>'
+            . '</html>';
 
-        preg_match_all($regexp, stripslashes($str), $regs, PREG_SET_ORDER);
+        $document = new DOMDocument('1.0', RCUBE_CHARSET);
+        @$document->loadHTML($html);
 
-        // convert attributes to an associative array (name => value)
-        if ($regs) {
-            foreach ($regs as $attr) {
-                $attrib[strtolower($attr[1])] = html_entity_decode($attr[3] . $attr[4]);
+        if ($node = $document->getElementsByTagName('div')->item(0)) {
+            foreach ($node->attributes as $name => $attr) {
+                $attrib[strtolower($name)] = $attr->nodeValue;
             }
         }
 
@@ -365,12 +398,12 @@ class html
 class html_inputfield extends html
 {
     protected $tagname = 'input';
-    protected $type = 'text';
+    protected $type    = 'text';
     protected $allowed = array(
         'type','name','value','size','tabindex','autocapitalize','required',
         'autocomplete','checked','onchange','onclick','disabled','readonly',
         'spellcheck','results','maxlength','src','multiple','accept',
-        'placeholder','autofocus',
+        'placeholder','autofocus','pattern',
     );
 
     /**
@@ -392,8 +425,9 @@ class html_inputfield extends html
     /**
      * Compose input tag
      *
-     * @param string $value Field value
+     * @param string $value  Field value
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($value = null, $attrib = null)
@@ -409,6 +443,7 @@ class html_inputfield extends html
         }
         // set type
         $this->attrib['type'] = $this->type;
+
         return parent::show();
     }
 }
@@ -433,9 +468,9 @@ class html_passwordfield extends html_inputfield
 class html_hiddenfield extends html
 {
     protected $tagname = 'input';
-    protected $type = 'hidden';
-    protected $fields_arr = array();
+    protected $type    = 'hidden';
     protected $allowed = array('type','name','value','onchange','disabled','readonly');
+    protected $fields  = array();
 
     /**
      * Constructor
@@ -456,7 +491,7 @@ class html_hiddenfield extends html
      */
     public function add($attrib)
     {
-        $this->fields_arr[] = $attrib;
+        $this->fields[] = $attrib;
     }
 
     /**
@@ -467,9 +502,10 @@ class html_hiddenfield extends html
     public function show()
     {
         $out = '';
-        foreach ($this->fields_arr as $attrib) {
+        foreach ($this->fields as $attrib) {
             $out .= self::tag($this->tagname, array('type' => $this->type) + $attrib);
         }
+
         return $out;
     }
 }
@@ -489,6 +525,7 @@ class html_radiobutton extends html_inputfield
      *
      * @param string $value  Value of the checked field
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -520,6 +557,7 @@ class html_checkbox extends html_inputfield
      *
      * @param string $value  Value of the checked field
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -553,6 +591,7 @@ class html_textarea extends html
      *
      * @param string $value  Textbox value
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -594,7 +633,7 @@ class html_textarea extends html
  * $select->add(array('Switzerland','Germany'), array('CH','DE'));
  *
  * // generate pulldown with selection 'Switzerland'  and return html-code
- * // as second argument the same attributes available to instanciate can be used
+ * // as second argument the same attributes available to instantiate can be used
  * print $select->show('CH');
  * </pre>
  *
@@ -632,6 +671,7 @@ class html_select extends html
      *
      * @param string $select Value of the selection option
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($select = array(), $attrib = null)
@@ -674,8 +714,8 @@ class html_table extends html
     protected $allowed = array('id','class','style','width','summary',
         'cellpadding','cellspacing','border');
 
-    private $header = array();
-    private $rows = array();
+    private $header   = array();
+    private $rows     = array();
     private $rowindex = 0;
     private $colindex = 0;
 
@@ -741,12 +781,12 @@ class html_table extends html
      * Remove a column from a table
      * Useful for plugins making alterations
      *
-     * @param string $class
+     * @param string $class Class name
      */
     public function remove_column($class)
     {
         // Remove the header
-        foreach ($this->header as $index=>$header){
+        foreach ($this->header as $index => $header){
             if ($header->attrib['class'] == $class){
                 unset($this->header[$index]);
                 break;
@@ -754,8 +794,8 @@ class html_table extends html
         }
 
         // Remove cells from rows
-        foreach ($this->rows as $i=>$row){
-            foreach ($row->cells as $j=>$cell){
+        foreach ($this->rows as $i => $row){
+            foreach ($row->cells as $j => $cell){
                 if ($cell->attrib['class'] == $class){
                     unset($this->rows[$i]->cells[$j]);
                     break;
@@ -775,7 +815,7 @@ class html_table extends html
         $this->colindex = 0;
         $this->rows[$this->rowindex] = new stdClass;
         $this->rows[$this->rowindex]->attrib = $attr;
-        $this->rows[$this->rowindex]->cells = array();
+        $this->rows[$this->rowindex]->cells  = array();
     }
 
     /**
@@ -822,6 +862,7 @@ class html_table extends html
      * Build HTML output of the table data
      *
      * @param array $attrib Table attributes
+     *
      * @return string The final table HTML code
      */
     public function show($attrib = null)
@@ -836,7 +877,7 @@ class html_table extends html
         if (!empty($this->header)) {
             $rowcontent = '';
             foreach ($this->header as $c => $col) {
-                $rowcontent .= self::tag($this->_col_tagname(), $col->attrib, $col->content);
+                $rowcontent .= self::tag($this->_head_tagname(), $col->attrib, $col->content);
             }
             $thead = $this->tagname == 'table' ? self::tag('thead', null, self::tag('tr', null, $rowcontent, parent::$common_attrib)) :
                 self::tag($this->_row_tagname(), array('class' => 'thead'), $rowcontent, parent::$common_attrib);
@@ -889,7 +930,16 @@ class html_table extends html
     private function _row_tagname()
     {
         static $row_tagnames = array('table' => 'tr', 'ul' => 'li', '*' => 'div');
-        return $row_tagnames[$this->tagname] ? $row_tagnames[$this->tagname] : $row_tagnames['*'];
+        return $row_tagnames[$this->tagname] ?: $row_tagnames['*'];
+    }
+
+    /**
+     * Getter for the corresponding tag name for table row elements
+     */
+    private function _head_tagname()
+    {
+        static $head_tagnames = array('table' => 'th', '*' => 'span');
+        return $head_tagnames[$this->tagname] ?: $head_tagnames['*'];
     }
 
     /**
@@ -898,7 +948,6 @@ class html_table extends html
     private function _col_tagname()
     {
         static $col_tagnames = array('table' => 'td', '*' => 'span');
-        return $col_tagnames[$this->tagname] ? $col_tagnames[$this->tagname] : $col_tagnames['*'];
+        return $col_tagnames[$this->tagname] ?: $col_tagnames['*'];
     }
-
 }
